---------------------------------------------------------------------------------------------
--- Step 1: Alter Views
---------------------------------------------------------------------------------------------


declare     SYSTEM_EXCEPTION EXCEPTION;     PRAGMA     EXCEPTION_INIT (SYSTEM_EXCEPTION, -12003); 
begin execute immediate   ' drop materialized view "TRAFFICBYDAY"'; 
EXCEPTION when SYSTEM_EXCEPTION then null; end;
/

CREATE MATERIALIZED VIEW "TRAFFICBYDAY" USING NO INDEX AS
SELECT
  TO_CHAR("VISITS"."STARTDATETIME", 'YYYYMMDD') AS "DATE",
	TO_CHAR(trunc(CAST( "VISITS"."STARTDATETIME" AS DATE),'mm'), 'YYYYMMDD') AS "MONTH",
  "VISITS"."TRAFFICTYPE" AS "TRAFFICTYPE",
  "VISITS"."CAMPAIGNID" AS "CAMPAIGNID",
  "PAGES"."ITEMID" AS "ITEMID",
  LOWER( SUBSTR( SUBSTR(TO_CHAR( "ITEMURLS"."URL" ),1,800), 1, INSTR( "ITEMURLS"."URL" || '?' , '?') - 1 ) ) AS "URL",
  "VISITS"."KEYWORDSID" AS "KEYWORDSID",
  LOWER( SUBSTR(TO_CHAR( "KEYWORDS"."TEXT" ),1,400) ) AS "KEYWORDS",
  "VISITS"."REFERRINGSITEID" AS "REFERRINGSITEID",
  LOWER( "REFERRINGSITES"."HOST" ) AS "REFERRINGSITE",
  LOWER( "VISITS"."MULTISITE" ) AS "MULTISITE",
  LOWER( "VISITS"."DEVICENAME" ) AS "DEVICENAME",
  LOWER( "VISITS"."LANGUAGE" ) AS "LANGUAGE",
  (CASE WHEN "VISITS"."VISITORVISITINDEX" = 1 THEN 1 ELSE 0 END) AS "FIRSTVISIT",
  COUNT( 1 ) AS "VISITS",
  SUM( "VISITS"."VALUE" ) AS "VALUE"
FROM
  "VISITS" 
  INNER JOIN "PAGES"  ON ("VISITS"."VISITID" = "PAGES"."VISITID") AND ("VISITS"."VISITORID" = "PAGES"."VISITORID") AND ("PAGES"."VISITPAGEINDEX" = 1) AND ("PAGES"."ITEMID" <> '00000000-0000-0000-0000-000000000000')
  INNER JOIN "KEYWORDS"  ON ("VISITS"."KEYWORDSID" = "KEYWORDS"."KEYWORDSID")
  INNER JOIN "ITEMURLS"  ON ("PAGES"."ITEMID" = "ITEMURLS"."ITEMID")
  INNER JOIN "REFERRINGSITES"  ON ("VISITS"."REFERRINGSITEID" = "REFERRINGSITES"."REFERRINGSITEID")
  INNER JOIN "VISITORS"  ON ("VISITS"."VISITORID" = "VISITORS"."VISITORID") AND ("VISITORS"."VISITORCLASSIFICATION" < 600)
GROUP BY
  TO_CHAR("VISITS"."STARTDATETIME", 'YYYYMMDD'),
	TO_CHAR(trunc(CAST( "VISITS"."STARTDATETIME" AS DATE),'mm'), 'YYYYMMDD'),
  "VISITS"."TRAFFICTYPE",
  "VISITS"."CAMPAIGNID",
  "PAGES"."ITEMID",
  LOWER( SUBSTR( SUBSTR(TO_CHAR( "ITEMURLS"."URL" ),1,800), 1, INSTR( "ITEMURLS"."URL" || '?' , '?') - 1 ) ),
  "VISITS"."KEYWORDSID",
  LOWER( SUBSTR(TO_CHAR( "KEYWORDS"."TEXT" ),1,400) ),
  "VISITS"."REFERRINGSITEID",
  LOWER( "REFERRINGSITES"."HOST" ),
  LOWER( "VISITS"."MULTISITE" ),
  LOWER( "VISITS"."DEVICENAME" ),
  LOWER( "VISITS"."LANGUAGE" ),
  (CASE WHEN "VISITS"."VISITORVISITINDEX" = 1 THEN 1 ELSE 0 END)
/


declare     SYSTEM_EXCEPTION EXCEPTION;     PRAGMA     EXCEPTION_INIT (SYSTEM_EXCEPTION, -12003); 
begin execute immediate   ' drop materialized view "VALUEBYSOURCE"'; 
EXCEPTION when SYSTEM_EXCEPTION then null; end;
/


CREATE MATERIALIZED VIEW "VALUEBYSOURCE" AS
SELECT
  TO_CHAR("VISITS"."STARTDATETIME", 'YYYYMMDD') AS "DATE",
	TO_CHAR(trunc(CAST( "VISITS"."STARTDATETIME" AS DATE),'mm'), 'YYYYMMDD') AS "MONTH",
  "VISITS"."TRAFFICTYPE" AS "TRAFFICTYPE",
  LOWER( "VISITS"."MULTISITE" ) AS "MULTISITE",
  LOWER( "VISITS"."DEVICENAME" ) AS "DEVICENAME",
  LOWER("VISITS"."LANGUAGE") "LANGUAGE",
  SUM( "VISITORS"."VISITCOUNT" ) AS "VISITS",
  SUM( "VISITORS"."VALUE" ) AS "VALUE",
  SUM("VISITS"."VALUE") "FIRSTVISITVALUE",
  COUNT( 1 ) AS "VISITORS"
FROM 
  "VISITORS" 
  INNER JOIN "VISITS"  ON ("VISITS"."VISITORVISITINDEX" = 1) AND ("VISITS"."VISITORID" = "VISITORS"."VISITORID")
  AND ("VISITORS".VisitorClassification < 600)
GROUP BY
  TO_CHAR("VISITS"."STARTDATETIME", 'YYYYMMDD'),
	TO_CHAR(trunc(CAST( "VISITS"."STARTDATETIME" AS DATE),'mm'), 'YYYYMMDD'),
  "TRAFFICTYPE",
  LOWER( "VISITS"."MULTISITE" ),
  LOWER( "VISITS"."DEVICENAME" ),
  LOWER( "VISITS"."LANGUAGE" )

/


declare     SYSTEM_EXCEPTION EXCEPTION;     PRAGMA     EXCEPTION_INIT (SYSTEM_EXCEPTION, -12003); 
begin execute immediate   ' drop materialized view "VISITEVENTS"'; 
EXCEPTION when SYSTEM_EXCEPTION then null; end;
/


CREATE MATERIALIZED VIEW "VISITEVENTS" AS
SELECT
  TO_CHAR("VISITS"."STARTDATETIME", 'YYYYMMDD') AS "DATE",
	TO_CHAR(trunc(CAST( "VISITS"."STARTDATETIME" AS DATE),'mm'), 'YYYYMMDD') AS "MONTH",
  "VISITS"."VISITID" AS "VISITID",
  "VISITS"."TRAFFICTYPE" AS "TRAFFICTYPE",
  "VISITS"."CAMPAIGNID" AS "CAMPAIGNID",
  LOWER( "VISITS"."MULTISITE" ) AS "MULTISITE",
  LOWER( "VISITS"."DEVICENAME" ) AS "DEVICENAME",
  LOWER( "VISITS"."LANGUAGE" ) AS "LANGUAGE",
  "VISITS"."VALUE" AS "VALUE",
  "PAGEEVENTS"."PAGEEVENTDEFINITIONID" AS "PAGEEVENTDEFINITIONID",
  "PAGEEVENTDEFINITIONS"."NAME" AS "PAGEEVENTNAME",
  "PAGEEVENTDEFINITIONS"."ISGOAL" AS "ISGOAL",
  "PAGES"."ITEMID" AS "ITEMID",
  "PAGEEVENTS"."ITEMID" AS "EVENTITEMID",
  LOWER( "PAGEEVENTS"."DATAKEY" ) AS "DATAKEY",
  COUNT( 1 ) AS "NUMBEROFEVENTS"
FROM 
  "PAGEEVENTS" 
  INNER JOIN "PAGES"  ON ("PAGEEVENTS"."PAGEID" = "PAGES"."PAGEID") AND ("PAGEEVENTS"."VISITID" = "PAGES"."VISITID") AND ("PAGEEVENTS"."VISITORID" = "PAGES"."VISITORID")
  INNER JOIN "VISITS"  ON ("VISITS"."VISITID" = "PAGEEVENTS"."VISITID") AND ("VISITS"."VISITORID" = "PAGEEVENTS"."VISITORID")
  INNER JOIN "PAGEEVENTDEFINITIONS"  ON ("PAGEEVENTS"."PAGEEVENTDEFINITIONID" = "PAGEEVENTDEFINITIONS"."PAGEEVENTDEFINITIONID")
  INNER JOIN "VISITORS"  ON ("VISITORS".VisitorId = "VISITS"."VISITORID") AND ("VISITORS"."VISITORCLASSIFICATION" < 600)
WHERE 
  ("PAGEEVENTS"."DATAKEY" > ' ') OR
  ("PAGEEVENTDEFINITIONS"."ISGOAL" = 1)
GROUP BY 
  TO_CHAR("VISITS"."STARTDATETIME", 'YYYYMMDD'),
	TO_CHAR(trunc(CAST( "VISITS"."STARTDATETIME" AS DATE),'mm'), 'YYYYMMDD'),
  "VISITS"."VISITID",
  "VISITS"."TRAFFICTYPE",
  "VISITS"."CAMPAIGNID",
  LOWER( "VISITS"."MULTISITE" ),
  LOWER( "VISITS"."DEVICENAME" ),
  LOWER( "VISITS"."LANGUAGE" ),
  "VISITS"."VALUE", 
  "PAGEEVENTS"."PAGEEVENTDEFINITIONID",
  "PAGEEVENTDEFINITIONS"."NAME",
  "PAGEEVENTDEFINITIONS"."ISGOAL",
  LOWER( "PAGEEVENTS"."DATAKEY" ),
  "PAGES"."ITEMID",
  "PAGEEVENTS"."ITEMID"



/

declare     SYSTEM_EXCEPTION EXCEPTION;     PRAGMA     EXCEPTION_INIT (SYSTEM_EXCEPTION, -12003); 
begin execute immediate   ' drop materialized view "VISITORSBYLOCATION"'; 
EXCEPTION when SYSTEM_EXCEPTION then null; end;
/


CREATE MATERIALIZED VIEW "VISITORSBYLOCATION" AS
SELECT
    TO_CHAR("STARTDATETIME", 'YYYYMMDD') AS "DATE",
    "VISITORS"."VISITORCLASSIFICATION" AS "VISITORCLASSIFICATION",
    "LOCATIONID" AS "LOCATIONID",
    "COUNTRY" AS "COUNTRY",
	"BUSINESSNAME" AS "BUSINESSNAME",
    "REGION" AS "REGION",
    "VISITS"."VISITORID" AS "VISITORID",
    COUNT( 1 ) AS "VISITS",
    SUM( "VISITS"."VALUE" ) AS "VALUE"
FROM
    "VISITS" 
    INNER JOIN "VISITORS"  ON ("VISITS"."VISITORID" = "VISITORS"."VISITORID") AND ("VISITORS"."VISITORCLASSIFICATION" < 600)
GROUP BY
    "VISITS"."VISITORID",
    "LOCATIONID",
    "COUNTRY",
    "BUSINESSNAME",
    "REGION",
    "VISITORCLASSIFICATION",
    TO_CHAR("STARTDATETIME", 'YYYYMMDD')


/


declare     SYSTEM_EXCEPTION EXCEPTION;     PRAGMA     EXCEPTION_INIT (SYSTEM_EXCEPTION, -12003); 
begin execute immediate   ' drop view "TRAFFICBYCAMPAIGN"'; 
EXCEPTION when SYSTEM_EXCEPTION then null; end;
/

CREATE VIEW "TRAFFICBYCAMPAIGN" AS
WITH "TBC" AS
(
  (
    SELECT
      "CAMPAIGNID" AS "CAMPAIGNID",
      SUM( "VISITS" ) AS "VISITS",
      SUM( "VALUE" ) AS "VALUE"
    FROM
      "TRAFFICBYDAY"
    WHERE
      ("CAMPAIGNID" IS NOT NULL) AND
      ("CAMPAIGNID" <> '00000000-0000-0000-0000-000000000000') AND
      ("CAMPAIGNID" <> '00000001-0000-0000-0000-000000000000')
    GROUP BY
      "CAMPAIGNID"
  )
  UNION ALL
  (
    SELECT
      "VISITS"."CAMPAIGNID" AS "CAMPAIGNID",
      COUNT( 1 ) AS "VISITS",
      SUM( "VISITS"."VALUE" ) AS "VALUE"
    FROM
      "VISITS" 
      INNER JOIN "VISITORS"  ON ("VISITORS".VisitorId = "VISITS"."VISITORID") AND ("VISITORS"."VISITORCLASSIFICATION" < 600)
    WHERE
      ("VISITS"."STARTDATETIME" > TO_DATE(NVL( (SELECT MAX(cache."DATE") FROM "TRAFFICBYDAY" cache), '0001-01-01' ), 'YYYYMMDD') ) AND
      ("CAMPAIGNID" IS NOT NULL) AND
      ("CAMPAIGNID" <> '00000000-0000-0000-0000-000000000000') AND
      ("CAMPAIGNID" <> '00000001-0000-0000-0000-000000000000')
    GROUP BY
      "VISITS"."CAMPAIGNID"
  )
)
SELECT
  "CAMPAIGNID" AS "CAMPAIGNID",
  SUM( "VISITS" ) AS "VISITS",
  SUM( "VALUE" ) AS "VALUE"
FROM
  "TBC"
GROUP BY
  "CAMPAIGNID"


/


exit
/